   ; ds01.asm 
   	org 0400h
   	ljmp main
;-------------------------------m2_tx-------------------------------------
m2_tx:
   	mov c,acc.7
	mov p1.5,c
	rl a
	jb 8,m2_tx_lsb	; If bit 8 is one TX LSB
   ; Otherwise Tx MSb
	djnz r2,m2_tx_exit
	mov a,054h	; Load LSB
	setb 8
	mov r2,#r2_init
	sjmp m2_tx_exit
m2_tx_lsb:
	djnz r2,m2_tx_exit
	setb 9 		; Tell to the main routine that Tx is finished
m2_tx_exit:
	reti
;-----------------------End of m2_tx ISR-------------------------------
;
;-------------------------------m2_rx----------------------------------
m2_rx:
	mov c,p1.3
	rlc a
	djnz r2,m2_rx_not_copy
	movx @dptr,a
	mov r2,#r2_init
	inc dptr
	setb tr0	; Turn T0 on
	reti
m2_rx_not_copy:
	clr tr0		; Stop T0
	mov th0,#th0_load
	mov tl0,#tl0_load
	reti
;-------------------------End of m2_rx ISR------------------------------
;
;-------------------------t0_m2rx--------------------------------------
t0_m2rx:
   ; This ISR for TO to be used with m2_rx
   	mov a,dpl	; DPL is actually how many bytes read
	mov 057h,a	; Copy it to memory so other routines will know how many 
			; bytes read
	clr tr0		; Stop T0
	setb p1.1	; Reset 6408
	setb 10		; Flag that reception of a frame is finished
	reti
	reti
;-------------------------End of t0_m2rx ISR-------------------------
;
;------------------------------t0_m5rx---------------------------------
t0_m5rx:
   ; This is an ISR for T0 to be used with m5_rx ISR
	setb 11		; Indicate that reception is finished
	clr p3.4	; Reset 6409
	clr tr0		; Turn T0 off
	mov 058h,dph	; Save address of the last byte written
	mov 059h,dpl
   	reti
;-----------------------------end of t0_m5rx ISR-----------------------------
;
;--------------------------------t0_acq--------------------------------------
t0_acq:
   ; This is an for T0 to be used within acq_cycle subroutine to control delay
   ; between Acquire and Send commands
	djnz 05ah,t0_acq_exit
	setb 13		; Inform waiting loop that cycle is finished
	clr tr0		; Stop T0
	reti

t0_acq_exit:
	clr tr0		; Stop T0
	mov th0,#th0_delay_acq	; Reload 
	mov tl0,#tl0_delay_acq
	setb tr0	; Start T0
	
   	reti
;--------------------------------end of t0_acq ISR----------------------------
;
;-----------------------------------t0_33----------------------------------
t0_33:
   ; This ISR for T0 is used within subroutine handling cycling acquisition
	djnz 060h,t0_33_exit
	setb 16	; Inform waiting loop that cycle is finished
	clr tr0		; Stop T0
	reti

t0_33_exit:
	clr tr0		; Stop T0
	mov th0,#0	; Reload 
	mov tl0,#0
	setb tr0	; Start T0
	
   	reti
;--------------------------------end of t0_acq ISR----------------------------
   
main:	
	mov tcon,#05h	; T1 off, T0 off, both external interrupts are falling edge triggered
			; T0 set by bootloader to 16 bit mode, leave it
	mov ie,#0	; Disable all interrupts
	mov ip,#05h	; Both external interrupts - priority 1, all the rest - 0
	setb p1.7	; Switch LED off
	mov 05fh,#20	; Setup default Acquisition cycle delay
	mov 05ah,#1	; Setup default delay between Acquire and Send commands
   ; Some calculation regarding timing
   ; With 16MHz clock one machine cycle takes 0.75microsec. If T0 initialized to 0 and used
   ; as 16 bit counter it will time out in 49.151millisec.
   	clr 14
	clr ri		; Whatever is received up to this point is discarded
	mov r0,#r0_init	; Init Serial Rx buffer pointer
main_loop:
   ; Serial port interrupts are disabled, we need to poll RI bit to see
   ; if a byte rang in
	jbc 14,main_program_exit
	jnb ri,main_loop	
	mov a,sbuf	 	; Copy received byte
	clr ri			; Acknowledge reception
	cjne a,#cr,main_copy_data	; Check if we received CR
   ;main_cr	
   ; 
   ; Calculate how many bytes received
   	clr c
	mov a,r0
	subb a,#r0_init
	dec a	; Don't include first byte
	mov 05bh,a	; Copy to memory
   ;We receiving ASCII data from the host PC and need to convert it to binary
   	lcall convert_to_binary
	
	lcall cmd_parse	; See what we received
	clr ri		; If something was received in between - discard it
	mov r0,#r0_init	; Reinit r0 to be ready for next command
	sjmp main_loop	; After cmd_parse returns go back to the main loop

main_program_exit:
	clr p1.7		; Switch LED on
	mov ie,#090h
	ret

	
main_copy_data:
   ; Just copy data to the buffer	
	mov @r0,a
	inc r0
	sjmp main_loop
	
cmd_parse:
   ; Parse what we've received from the host
   	mov r0,#r0_init
   	cjne @r0,#030h,cmd_parse_check_31
   ; Send Acquisition commands is received from the host
   	clr 15	; Tx both Acq and Send commands
	lcall acq_cycle
	ret
cmd_parse_check_31:
   ; Check if Non acqusition command received
   	cjne @r0,#031h,cmd_parse_check_32
   ; We need to send Non Acquisition command to the tool
   	mov r1,#r1_init		
	inc r1			; First byte isn used
	mov 061h,@r1		; Load software counter
	mov a,061h
	jz cmd_31_exit		; No words to Tx
cmd_31_start:
   	inc r1
	mov 053h,@r1	; Load MSB
	inc r1
	mov 054h,@r1	; Load LSB
	inc r1
	cjne @r1,#0,cmd_31_data_sync
	setb 12		; Command sync
	sjmp cmd_31_after_sync
cmd_31_data_sync:
	clr 12		; Data sync
cmd_31_after_sync:
	lcall send_cmd
	mov 062h,#50
cmd_31_after_cmd_wait:
	djnz 062h,cmd_31_after_cmd_wait

	djnz 061h,cmd_31_start
   ; Otherwise everythins has been transmitted
   
cmd_31_exit:
	ret	; End of the routine parsing command 31

	
cmd_parse_check_32:
  	cjne @r0,#032h,cmd_parse_check_33
	setb 15		;Tx Send command only
	lcall acq_cycle
	ret
cmd_parse_check_33:
	cjne @r0,#033h,cmd_parse_check_34
   ; We need to start cyclic acqusition	
   ;
   ; Send/Acq sequence should be programmed by Acq cycle delay command
   ; issued prior to it, so we'll just call acq_cycle subroutine without setting bit 15
cmd_parse_check_33_start:
	mov 060h,05fh	; Copy delay constant
	mov a,060h
	jz cmd_33_exit	; If delay constant is zero we cannot operate
   	lcall acq_cycle
   ; After first acquisition cycle we need to wait for some time
   ; Align T0 ISR
   	mov dptr,#t0_33
	mov 050h,dph
	mov 051h,dpl
	mov dptr,#0bh
	mov a,#02h
	movx @dptr,a
	inc dptr
	mov a,050h
	movx @dptr,a
	inc dptr
	mov a,051h
	movx @dptr,a
	
	clr tr0		; Stop T0
	mov th0,#0	; Preload T0
	mov tl0,#0	
	mov ie,#082h	; Enable T0 interrupts
	setb tr0	; Start T0
	clr 16		; Before entering waiting loop clear bit 16
cmd_33_wait_t0:
	jnb 16,cmd_33_wait_t0	; Wait until t0_33 ISR sets bit 16
	mov ie,#0	; Disable all interrupts
   ; First cycle is passed through. Now every cycle we'll be asking host what
   ; to send to the tool
	clr ti
	clr ri		; We'll expect return from the host
	mov sbuf,#034h
cmd_33_sbuf_wait_34:
	jnb ti,cmd_33_sbuf_wait_34
	clr ti
   ; Reception of a byte (any except from q) will be the signal
   ;to start next acquisition cycle
cmd_33_sbuf_wait_ri:
	jnb ri,cmd_33_sbuf_wait_ri
	mov a,sbuf
	clr ri
	cjne a,#071h,cmd_parse_check_33_start	; Jump to the next cycle if not q
cmd_33_exit:
	ret

	
cmd_parse_check_34:
	cjne @r0,#034h,cmd_parse_check_quit
   ; Acquisition cycle delay needs to be setup
	mov r1,#r1_init
	inc r1		; First byte isn't used
	mov 05fh,@r1
	inc r1
	cjne @r1,#0,cmd_parse_check_34_acq_send
   ; During first Acquisition Cycle we need to send only Send command
   	setb 15
	sjmp cmd_parse_check_34_exit
cmd_parse_check_34_acq_send:
   ; During first Acquisition Cycle we need to send both Acq and Send commands
   	clr 15
cmd_parse_check_34_exit:	
   	ret
	
cmd_parse_check_quit:
	cjne @r0,#071h,cmd_parse_error
   ; Quit the program
   	setb 14		; Signal to the main loop that Quit signal is received
	ret
cmd_parse_error:
   ; We need to inform that did not realize what the command is
   	clr ti
   	mov sbuf,#033h
cmd_parse_error_wait:
	jnb ti,cmd_parse_error_wait
	clr ti
	ret
	
;-----------------------------Functions----------------------------------
;
;-----------------------send_cmd-----------------------------------------
send_cmd:
   ; This function sends a command to the tool
   ;
   ; Must be passed by calling function:
   ; 053h MSB to Tx
   ; 054h LSB to Tx
   ; bit 12 - If set Tx with Command Sync, if cleared - with Data Sync
   ;
   ; Realign interrupt vector to EX0 Int to point to m2_tx ISR
	setb p1.1	; Set Master reset
	clr p1.4	; Disable Encoder
   	clr p3.4	; Reset 6409
	
	mov dptr,#m2_tx
	mov 050h,dph
	mov 051h,dpl
	mov dptr,#03h
	mov a,#02h
	movx @dptr,a
	inc dptr
	mov a,050h
	movx @dptr,a
	inc dptr
	mov a,051h
	movx @dptr,a
   ; Send two bytes. MSB in acc, LSB in b
   	jnb 12,send_cmd_data_sync
   	setb p1.6	; SS  high - Command sync
	sjmp send_cmd_after_sync
send_cmd_data_sync:	
	clr p1.6	; SS low - Data sync
send_cmd_after_sync:
	setb p1.1	; Set Master reset
	clr p1.4	; Disable Encoder
	mov ie,#081h	; EX0 Enable
	clr p1.1	; Remove Reset
send_cmd_start:
   ; MSB to send is stored at 053h, LSB - at 054h
   	mov a,053h
	clr 8 		; Tx MSB
	mov r2,#r2_init	; Init r2, will be used as bit counter in the ISR
	clr 9
	setb p1.4	; Enable Encoder
			; Tx ISRs will be called automatically as Tx will start
send_cmd_wait:
	jnb 9,send_cmd_wait	; Wait when m2_tx will finish
	mov ie,#0		; Disable all interrupts
	clr p1.4		; Disable Encoder
send_cmd_exit:
	ret
;-----------------------End of send_cmd---------------------------------------
;
;-----------------------------m2_data----------------------------------------
m2_data:
   ; This subroutine handles reception of M2 data
   ; Realign interrupt vector to EX0 Int to point to m2_rx ISR
   ; After acquisition of a frame is finished it's sent to the host PC as binary data
   	clr p3.4	; Reset 6409
   	mov dptr,#m2_rx
	mov 050h,dph
	mov 051h,dpl
	mov dptr,#03h
	mov a,#02h
	movx @dptr,a
	inc dptr
	mov a,050h
	movx @dptr,a
	inc dptr
	mov a,051h
	movx @dptr,a
	
   ;Realign interrupt vector for T0
   	mov dptr,#t0_m2rx
	mov 050h,dph
	mov 051h,dpl
	mov dptr,#0bh
	mov a,#02h
	movx @dptr,a
	inc dptr
	mov a,050h
	movx @dptr,a
	inc dptr
	mov a,051h
	movx @dptr,a

	clr p1.4	; Clear EE
	mov r2,#r2_init
	mov dptr,#m2_buf
	mov ie,#083h	; Enable EX0 and T0 Ints
	clr tr0		; Stop T0
	mov th0,#th0_load	; Load T0 in case of timeout
	mov tl0,#tl0_load	
			; if data is not received before this time
	setb tr0	; Start T0
	clr p1.1	; Remove reset condition, if was previously set
	clr 10
m2_data_wait:
	jnb 10,m2_data_wait	; Wait until reception of a frame is finished
	mov ie,#0	; Disable all interrupts
   ; Now send all this binary data over serial link to the host PC
   ; First check if there is any data at all
   	mov a,057h	; Number of bytes read
	jz m2_data_to
   ; Otherwise send everything to PC in binary
   ; First byte is status byte 030h - some data received:
   	clr ti
	mov sbuf,#030h
m2_data_wait_zero:
	jnb ti,m2_data_wait_zero
	clr ti

   	mov dptr,#m2_buf	; Point DPTR to the beginning of M2_buf
m2_data_tx:
	movx a,@dptr		; Read one byte
	inc dptr
	mov sbuf,a
m2_data_sbuf:
	jnb ti,m2_data_sbuf
	clr ti
	djnz 057h,m2_data_tx
	ljmp m2_data_exit	; Everything is transmitted - exit
m2_data_to:
   ; Time out occures. Some data was expected but nothing received
   ; We need to somehow tell this to PC
   	clr ti
	mov sbuf,#031h
m2_data_to_wait_one:
	jnb ti,m2_data_to_wait_one
	clr ti
   	ljmp m2_data_exit
	
m2_data_exit:
	ret
;-----------------------------End of m2_data----------------------------------
;
;---------------------------------m5_data---------------------------------------
m5_data:
   ; This subroutine handles acquisition of M5 data

	clr p3.4		; Force reset of 6409
	
   ; Initialize M5 ISR. To exclude jump instruction we'll put this handler directly
   ; to lower addresses as binary
	mov r5,#0	
	mov r6,#m5_isr_vector
m5_data_init_isr:
	mov dptr,#m5_rx
	mov a,r5
	movc a,@a+dptr
	jz m5_data_init_isr_done
	inc r5
	mov dph,#0
	mov dpl,r6
	movx @dptr,a
	inc r6
	sjmp m5_data_init_isr
	
m5_data_init_isr_done:

   ; Init T0 ISR
   	mov dptr,#t0_m5rx
	mov 050h,dph
	mov 051h,dpl
	mov dptr,#0bh
	mov a,#02h
	movx @dptr,a
	inc dptr
	mov a,050h
	movx @dptr,a
	inc dptr
	mov a,051h
	movx @dptr,a
	
	mov dptr,#m5_buf	; Init M5 buffer for M5 ISR
	mov r2,#r2_init
	clr tr0			; Stop T0
	mov th0,#0		; Init T0. If no data received within 50millisec it will
	mov tl0,#0		; time out indicating that a tool is not responding
   ; Otherwise it will be reset every time M5 ISR is called. After frame reception is finished
   ; it will time out too but the difference is the number of bytes read.
   ; T0 ISR counts how many bytes read 058h-dph, 059h-dpl. 
	setb tr0			; Start T0
	mov ie,#086h		; Enable EX1 and T0 interrupts
	setb p3.4		; Remove reset
	clr 11
m5_data_wait:
	jnb 11,m5_data_wait	; Wait until reception is finished
	mov ie,#0		; When reception is finished disable all interrupts
   ; Now we need to send all the data to the host PC in binary format
   ; First check if there is any data at all
	mov dptr,#m5_buf
   	mov a,dpl
	cjne a,059h,m5_data_tx	; There is some data
	mov a,dph	; Even if dpl and 059h are equal we still need to check dph
	cjne a,058h,m5_data_tx	; There is some data - quite a lot
	ljmp m5_data_to	; T/O occurred

m5_data_tx:
   ; We need to send all the data as binary
	clr ti
   ; First byte sent is the status - 030h (0 ASCII) indicating that there is some data received
   	mov sbuf,#030h
m5_data_wait_zero:
	jnb ti,m5_data_wait_zero
	clr ti
	
m5_data_tx_start:
	movx a,@dptr
	inc dptr
	mov sbuf,a
	
	mov a,dpl
	cjne a,059h,m5_data_sbuf	; Still some data
	mov a,dph
	cjne a,058h,m5_data_sbuf	; Still some data
m5_data_sbuf_last_byte:		;Tx-ed byte was the last one
	jnb ti,m5_data_sbuf_last_byte
	clr ti
	ljmp m5_data_exit	; Everything is transmitted - exit

m5_data_sbuf: 	
	jnb ti,m5_data_sbuf
	clr ti
	sjmp m5_data_tx_start 	; There is for sure more data to Tx
	
m5_data_to:
   ; We expected some data and not received anything
   ; Host PC should be informed that T/O occurred
   ; Send to host 031h (1 ASCII) indicating that T/O occured
   	clr ti
	mov sbuf,#031h
m5_data_to_wait_one:
	jnb ti,m5_data_to_wait_one
	clr ti
	
   	ljmp m5_data_exit
	
m5_data_exit:
	ret
;-------------------------------End of m5_data----------------------------------
;
;---------------------------------acq_cycle-------------------------------------
acq_cycle:
   ; This subroutine controls acquisition cycle, i.e. when acqusition command is
   ; received from host it performs acquisition cycle - sends Acquire command
   ; waits for some time, sends Send command and if some data is received sends it
   ; to the host PC
   ;
   ; T0 is used to program delay between Acq and Send commands
   ; Init T0 ISR
   	clr p1.7		; Switch LED on
   	mov dptr,#t0_acq
	mov 050h,dph
	mov 051h,dpl
	mov dptr,#0bh
	mov a,#02h
	movx @dptr,a
	inc dptr
	mov a,050h
	movx @dptr,a
	inc dptr
	mov a,051h
	movx @dptr,a
   ; Read first word to send and send it
   	mov r1,#r1_init
	inc r1		 	;First byte isn't used here
	jb 15,acq_cycle_only_send	; If this bit is set we need to send Send command only
	mov 053h,@r1		; MSB to Tx
	inc r1
	mov 054h,@r1		; LSB to Tx
	inc r1
	setb 12			; Command sync
	lcall send_cmd		; Send command
	
	mov 05ah,@r1		; Load T0 with delay constant
	inc r1
	mov a,05ah
	jz acq_cycle_zero; If delay constant is zero - min delay
	sjmp acq_cycle_not_zero
acq_cycle_zero:
	mov 062h,#030h
acq_cycle_zero_loop:
	djnz 062h,acq_cycle_zero_loop
	sjmp acq_cycle_only_send
acq_cycle_not_zero:	
	clr tr0
	mov th0,#th0_delay_acq		; To get 1ms delay
	mov tl0,#tl0_delay_acq
	clr 13
	mov ie,#082h		; Enable T0 interrupts
	setb tr0		; After send_cmd returns wait for some time
acq_cycle_wait_t0:
	jnb 13,acq_cycle_wait_t0
	mov ie,#0h		; Disable all interrupts
   ; After delay we need to send second word
acq_cycle_only_send:
   	mov 053h,@r1		; MSB to Tx
	inc r1
	mov 054h,@r1		; LSB to Tx
	inc r1
	setb 12 		; Command sync
	lcall send_cmd
   ; Now we need to find out whether to receive M2 or M5
   	cjne @r1,#0,acq_cycle_m5
   ; acq_cycle_m2
	lcall m2_data
	sjmp acq_cycle_exit
acq_cycle_m5:
	lcall m5_data
	
acq_cycle_exit:
	setb p1.7	; Switch LED off
	ret
;-------------------------------End of acq_cycle--------------------


	
    $include(inc2.asm)
    $include(header02.h)
	end
